@@ -30,8 +30,8 @@ Features include:
 <li>Supports databases up to 2 terabytes
     (2<sup><small>41</small></sup> bytes) in size.</li>
 <li>Sizes of strings and BLOBs limited only by available memory.</li>
-<li>Small code footprint: less than 30K lines of C code,
-    less than 250KB code space (gcc on i486)</li>
+<li>Small code footprint: less than 250KiB fully configured or less
+    than 150KiB with optional features omitted.</li>
 <li><a href="speed.html">Faster</a> than popular client/server database
     engines for most common operations.</li>
 <li>Simple, easy to use <a href="capi3.html">API</a>.</li>
@@ -66,9 +66,21 @@ proc newsitem {date title text} {
   puts "<hr width=\"50%\">"
 }
 
-newsitem {2005-Aug-28} {Version 3.2.5} {
-  This release fixes a bug that causes DELETEs and UPDATEs to fail if
-  they attempt to changes more than 40960 rows.
+newsitem {2005-Sep-16} {Version 3.2.6 - Critical Bug Fix} {
+  This version fixes a bug that can result in database
+  corruption if a VACUUM of a 1 gibibyte or larger database fails
+  (perhaps do to running out of disk space or an unexpected power loss)
+  and is later rolled back.
+  <p>
+  Also in this release:
+  The ORDER BY and GROUP BY processing was rewritten to use less memory.
+  Support for COUNT(DISTINCT) was added.  The LIKE operator can now be
+  used by the optimizer on columns with COLLATE NOCASE.
+}
+
+newsitem {2005-Aug-27} {Version 3.2.5} {
+  This release fixes a few more lingering bugs in the new code.
+  We expect that this release will be stable and ready for production use.
 }
 
 newsitem {2005-Aug-24} {Version 3.2.4} {
@@ -95,4 +107,4 @@ puts {
 <p align="right"><a href="oldnews.html">Old news...</a></p>
 </td></tr></table>
 }
-footer {$Id: index.tcl,v 1.120 2005/08/27 17:06:16 drh Exp $}
+footer {$Id: index.tcl,v 1.125 2005/09/17 15:17:40 drh Exp $}
